"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeConfig = exports.validateConfig = exports.migrateUserConfig = exports.createBackup = exports.resolveConfigPathByNetworkID = exports.getConfig = exports.getLogLevel = exports.isBinaryBuild = void 0;
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const fs_1 = require("fs");
const child_process_1 = require("child_process");
const path_1 = require("path");
const lisk_validator_1 = require("@liskhq/lisk-validator");
const lisk_framework_1 = require("lisk-framework");
const lisk_utils_1 = require("@liskhq/lisk-utils");
const constants_1 = require("../constants");
const debug = (0, debug_1.default)('lisk:migrator');
const LOG_LEVEL_PRIORITY = Object.freeze({
    FATAL: 0,
    ERROR: 1,
    WARN: 2,
    INFO: 3,
    DEBUG: 4,
    TRACE: 5,
});
const isBinaryBuild = (corePath) => (0, fs_1.existsSync)((0, path_1.join)(corePath, '.build'));
exports.isBinaryBuild = isBinaryBuild;
const getLogLevel = (loggerConfig) => {
    const highestLogPriority = Math.max(LOG_LEVEL_PRIORITY[String(loggerConfig.fileLogLevel).toUpperCase()], LOG_LEVEL_PRIORITY[String(loggerConfig.consoleLogLevel).toUpperCase()]);
    const [logLevel] = Object.entries(LOG_LEVEL_PRIORITY).find(([, v]) => v === highestLogPriority);
    return logLevel.toLowerCase();
};
exports.getLogLevel = getLogLevel;
const getConfig = async (corePath, customConfigPath) => {
    var _a;
    const command = [];
    const [network] = (0, fs_1.readdirSync)(`${corePath}/config`);
    let compiledConfigPath = `${corePath}/config/${network}/config.json`;
    command.push(`cd ${corePath}`);
    if ((0, exports.isBinaryBuild)(corePath)) {
        command.push('&& source env.sh');
    }
    if (customConfigPath) {
        command.push(`--config ${customConfigPath}`);
        compiledConfigPath = customConfigPath;
    }
    const fullCommand = command.join(' ');
    debug(`Core path: ${corePath}`);
    debug(`Cmd: ${fullCommand}`);
    cli_ux_1.default.action.start('Compiling Lisk Core configuration');
    (0, child_process_1.execSync)(fullCommand, { shell: '/bin/bash' });
    cli_ux_1.default.action.stop();
    cli_ux_1.default.action.start('Loading Lisk Core configuration');
    const config = await (_a = compiledConfigPath, Promise.resolve().then(() => tslib_1.__importStar(require(_a))));
    cli_ux_1.default.action.stop();
    return config;
};
exports.getConfig = getConfig;
const resolveConfigPathByNetworkID = async (networkIdentifier) => {
    const network = constants_1.NETWORK_CONSTANT[networkIdentifier].name;
    const configFilePath = (0, path_1.join)(__dirname, '../..', `config/${network}/config.json`);
    return configFilePath;
};
exports.resolveConfigPathByNetworkID = resolveConfigPathByNetworkID;
const createBackup = async (config) => {
    const backupPath = (0, path_1.join)(__dirname, '../..', 'backup');
    (0, fs_1.mkdirSync)(backupPath, { recursive: true });
    (0, fs_1.writeFileSync)((0, path_1.resolve)(`${backupPath}/config.json`), JSON.stringify(config, null, '\t'));
};
exports.createBackup = createBackup;
const migrateUserConfig = async (configV3, configV4, snapshotHeight) => {
    var _a;
    cli_ux_1.default.action.start('Starting migration of custom config properties.');
    if (!configV4.system.version) {
        cli_ux_1.default.action.start(`Setting config property 'system.version' to: ${constants_1.DEFAULT_VERSION}.`);
        configV4.system.version = constants_1.DEFAULT_VERSION;
        cli_ux_1.default.action.stop();
    }
    if (configV3.rootPath) {
        cli_ux_1.default.action.start(`Setting config property 'system.dataPath' to: ${configV3.rootPath}.`);
        configV4.system.dataPath = configV3.rootPath;
        cli_ux_1.default.action.stop();
    }
    if (configV3.logger) {
        const logLevel = (0, exports.getLogLevel)(configV3.logger);
        cli_ux_1.default.action.start(`Setting config property 'system.logLevel' to: ${logLevel}.`);
        configV4.system.logLevel = logLevel;
        cli_ux_1.default.action.stop();
    }
    if (configV3.transactionPool) {
        if (configV3.transactionPool.maxTransactions) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.maxTransactions' to: ${configV3.transactionPool.maxTransactions}.`);
            configV4.transactionPool
                .maxTransactions = configV3.transactionPool.maxTransactions;
            cli_ux_1.default.action.stop();
        }
        if (configV3.transactionPool.maxTransactionsPerAccount) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.maxTransactionsPerAccount' to: ${configV3.transactionPool.maxTransactionsPerAccount}.`);
            configV4.transactionPool
                .maxTransactionsPerAccount = configV3.transactionPool.maxTransactionsPerAccount;
            cli_ux_1.default.action.stop();
        }
        if (configV3.transactionPool.transactionExpiryTime) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.transactionExpiryTime' to: ${configV3.transactionPool.transactionExpiryTime}.`);
            configV4.transactionPool
                .transactionExpiryTime = configV3.transactionPool.transactionExpiryTime;
            cli_ux_1.default.action.stop();
        }
        if (configV3.transactionPool.minEntranceFeePriority) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.minEntranceFeePriority' to: ${configV3.transactionPool.minEntranceFeePriority}.`);
            configV4.transactionPool
                .minEntranceFeePriority = configV3.transactionPool.minEntranceFeePriority;
            cli_ux_1.default.action.stop();
        }
        if (configV3.transactionPool.minReplacementFeeDifference) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.minReplacementFeeDifference' to: ${configV3.transactionPool.minReplacementFeeDifference}.`);
            configV4.transactionPool
                .minReplacementFeeDifference = configV3.transactionPool.minReplacementFeeDifference;
            cli_ux_1.default.action.stop();
        }
    }
    if ((_a = configV3.rpc) === null || _a === void 0 ? void 0 : _a.mode) {
        cli_ux_1.default.action.start(`Setting config property 'rpc.modes' to: ${configV3.rpc.mode}.`);
        configV4.rpc.modes = [configV3.rpc.mode];
        cli_ux_1.default.action.stop();
    }
    if (configV3.network) {
        if (configV3.network.port) {
            cli_ux_1.default.action.start(`Setting config property 'network.port' to: ${configV3.network.port}.`);
            configV4.network.port = configV3.network.port;
            cli_ux_1.default.action.stop();
        }
        if (configV3.network.hostIp) {
            cli_ux_1.default.action.start(`Setting config property 'network.host' to: ${configV3.network.hostIp}.`);
            configV4.network.host = configV3.network.hostIp;
            cli_ux_1.default.action.stop();
        }
        if (configV3.network.maxOutboundConnections) {
            cli_ux_1.default.action.start(`Setting config property 'network.maxOutboundConnections' to: ${configV3.network.maxOutboundConnections}.`);
            configV4.network.maxOutboundConnections = configV3.network.maxOutboundConnections;
            cli_ux_1.default.action.stop();
        }
        if (configV3.network.maxInboundConnections) {
            cli_ux_1.default.action.start(`Setting config property 'network.maxInboundConnections' to: ${configV3.network.maxInboundConnections}.`);
            configV4.network.maxInboundConnections = configV3.network.maxInboundConnections;
            cli_ux_1.default.action.stop();
        }
        if (configV3.network.wsMaxPayload) {
            cli_ux_1.default.action.start(`Setting config property 'network.wsMaxPayload' to: ${configV3.network.wsMaxPayload}.`);
            configV4.network.wsMaxPayload = configV3.network.wsMaxPayload;
            cli_ux_1.default.action.stop();
        }
        if (configV3.network.advertiseAddress) {
            cli_ux_1.default.action.start(`Setting config property 'network.advertiseAddress' to: ${configV3.network.advertiseAddress}.`);
            configV4.network.advertiseAddress = configV3.network.advertiseAddress;
            cli_ux_1.default.action.stop();
        }
    }
    cli_ux_1.default.action.start("Calculating and updating config property 'genesis.minimumCertifyHeight'.");
    configV4.genesis.minimumCertifyHeight =
        snapshotHeight +
            1 +
            (constants_1.POS_INIT_ROUNDS + constants_1.NUMBER_ACTIVE_VALIDATORS - 1) *
                (constants_1.NUMBER_ACTIVE_VALIDATORS + constants_1.NUMBER_STANDBY_VALIDATORS);
    cli_ux_1.default.action.stop();
    if (configV4.modules.pos && !configV4.modules.pos.maxBFTWeightCap) {
        cli_ux_1.default.action.start(`Setting config property 'modules.pos.maxBFTWeightCap' to: ${constants_1.MAX_BFT_WEIGHT_CAP}.`);
        configV4.modules.pos.maxBFTWeightCap = constants_1.MAX_BFT_WEIGHT_CAP;
        cli_ux_1.default.action.stop();
    }
    cli_ux_1.default.action.stop();
    return configV4;
};
exports.migrateUserConfig = migrateUserConfig;
const validateConfig = async (config) => {
    try {
        const mergedConfig = lisk_utils_1.objects.mergeDeep({}, lisk_framework_1.applicationConfigSchema.default, config);
        (await lisk_validator_1.validator.validate(lisk_framework_1.applicationConfigSchema, mergedConfig));
        return true;
    }
    catch (error) {
        return false;
    }
};
exports.validateConfig = validateConfig;
const writeConfig = async (config, outputDir) => {
    if (!(0, fs_1.existsSync)(outputDir)) {
        (0, fs_1.mkdirSync)(outputDir, { recursive: true });
    }
    (0, fs_1.writeFileSync)((0, path_1.resolve)(outputDir, 'config.json'), JSON.stringify(config));
};
exports.writeConfig = writeConfig;
//# sourceMappingURL=config.js.map